local dialog = nil
local vb = nil

-- Read from the manifest.
class "RenoiseScriptingTool" (renoise.Document.DocumentNode)
  function RenoiseScriptingTool:__init()    
    renoise.Document.DocumentNode.__init(self) 
    self:add_property("Name", "Untitled Tool")
    self:add_property("Id", "Unknown Id")
  end

local manifest = RenoiseScriptingTool()
local ok,err = manifest:load_from("manifest.xml")
local tool_name = manifest:property("Name").value
local tool_id = manifest:property("Id").value


-------------------------------------------------------------------------------
-- Main
-------------------------------------------------------------------------------




-------------------------------------------------------------------------------
-- GUI
-------------------------------------------------------------------------------

local SLIDER_NAME_WIDTH = 40
local SLIDER_WIDTH = 180
local SLIDER_LABEL_WIDTH = 40
local NOTE_WIDTH = 200
local NOTE_HEIGHT = 64
local GROOVE_RATIO = 2 / 3

local slidervalues = { 0, 0, 0, 0 }

local function update()

  for s = 1, 4 do
  
    local groove = slidervalues[s]
    
    local label = vb.views["sliderlabel" .. tostring(s)]
    label.text = string.format("%d %%", groove)
    
    local id = 1 + ((s - 1) * 2)
    local note1 = vb.views["note" .. tostring(id)]
    local note2 = vb.views["note" .. tostring(id + 1)]
      
    local scale = groove / 100
    local diff = math.floor((NOTE_HEIGHT * scale * GROOVE_RATIO) + 0.5)
    local h1 = NOTE_HEIGHT + diff
    local h2 = NOTE_HEIGHT - diff
    note1.height = h1
    note2.height = h2
    
    local delay = math.floor((256 * scale * GROOVE_RATIO) + 0.5)
    note2.text = string.format("Note delay: %.2X", delay)
    
  end
    
end

local function setslider(s, value)
  slidervalues[s] = value
  update()
end

local function show_dialog()

  if dialog and dialog.visible then
    dialog:show()
    return
  end
  
  vb = renoise.ViewBuilder()
    
  local sliders = vb:column { spacing = 5 }
  
  for s = 1, 4 do
       
    local row = vb:row {
    
      vb:text {
        text = string.format("%d & %d", s - 1, s),
        width = SLIDER_NAME_WIDTH
      },
      
      vb:slider {
        id = "slider" .. tostring(s),
        min = 0,
        max = 100,
        value = 0,
        width = SLIDER_WIDTH,
        notifier = function(value)
          setslider(s, value)
        end
      },
      
      vb:text {
        id = "sliderlabel" .. tostring(s),
        width = SLIDER_LABEL_WIDTH,
        text = "0%"
      }
      
    }
    
    sliders:add_child(row)
  
  end
  
  local noteheads = vb:column {} 
  local notes = vb:column {}
  
  for n = 1, 8 do
    
    local notehead = vb:button {
      id = "notehead" .. tostring(n),
      width = 30,
      height = NOTE_HEIGHT,
      text = string.format("%.02d", n - 1)
    }
    
    noteheads:add_child(notehead)
    
    local note = vb:button {
      id = "note" .. tostring(n),
      width = NOTE_WIDTH,
      height = NOTE_HEIGHT,
      text = "Note delay: 00"
    }
    
    notes:add_child(note)
    
  end
  
  local content = vb:row {
    margin = 10,
    spacing = 20,
        
    vb:column { 
      vb:text {
        text = "16th notes + delay column values."
      },
      vb:row {
        noteheads,
        notes
      }
    },
    
    vb:column {
      spacing = 5,
      vb:text {
        text = "Groove sliders."
      },
      sliders,
    },
  }
  
  dialog = renoise.app():show_custom_dialog(tool_name, content)  

end


-------------------------------------------------------------------------------
-- Menu entries.
-------------------------------------------------------------------------------

renoise.tool():add_menu_entry {
  name = "Main Menu:Tools:" .. tool_name .. "...",
  invoke = show_dialog  
}


-------------------------------------------------------------------------------
-- Key bindings.
-------------------------------------------------------------------------------

--[[
renoise.tool():add_keybinding {
  name = "Global:Tools:" .. tool_name .. "...",
  invoke = show_dialog
}
--]]


-------------------------------------------------------------------------------
-- MIDI mappings.
-------------------------------------------------------------------------------

--[[
renoise.tool():add_midi_mapping {
  name = tool_id..":Show Dialog...",
  invoke = show_dialog
}
--]]


-- Debug function.
_AUTO_RELOAD_DEBUG = function()
  show_dialog()
end
